
// This example reads and filter datta from sensors MAX30102 or MAX30105
// The raw IR and filtered IR values are sent to the ArduHAL application
// The raw IR goes to Pin D0
// The filtered IR goes to Pin D1
// In the ArduHAl Pins 0 and 1 are configured as Gen_in_16

#include <Theremino.h>
#include <ThereminoFilters.h>
#include <Wire.h>
#include "MAX30102.h"

MAX30102 sensor;   

void setup() 
{  
    // ----------------------------------------------------------------
    // Calling setAsyncMode() the timer0 compare interrupt is used.
    // The ArduHAL exchange frequency is independent by the "loop()"
    // but the max frequency drops from about 300 to about 200 fps
    // ----------------------------------------------------------------
    //Theremino.setAsyncMode();
	
    // ----------------------------------------------------------------
    //  This baud rate must be the same as in the ArduHAL CommOptions
    //  Any baud rate from 1000 to 1000000 is valid
    //  More used baud rates are : 500000 250000 115200
    // ----------------------------------------------------------------
    Serial.begin(500000);
	
    // ----------------------------------------------------------------
    //  This part is required for MAX3010x sensor
    // ----------------------------------------------------------------
    //  We use the IR LED only because it provides the more stable data
    //  but we light also the RED LED to see some light
    // ----------------------------------------------------------------
    Wire.begin();                       // Initialize I2C communication
    sensor.begin();                     // Initialize the sensor
    sensor.setLEDs(60, 60, 0);          // RED, IR and GRN LEDs - Using IR only
    sensor.setPulseWidth(pw118);        // PulseWidth = 118 uS (adc 16 bit)
    sensor.setSampleRate(sr1000);       // SampleRate = 1000 samples per second
    sensor.setWorkingMode(wmHeartRate); // wmHeartRate (red) / wmSpO2 (red and IR) / wmMultiLed (RED, IR, GRN)
    sensor.setAdcRange(rge16384);       // adc range = max (16 uA)
    //sensor.setSamplingAverage(avg1);  // average (1 to 32) unused
}

// ------------------------------------------------- Filters declarations - HiPass 0.7 Hz
Filter hipass1(0.7, true);
Filter hipass2(0.7, true);
Filter hipass3(0.7, true);
Filter hipass4(0.7, true);
// ------------------------------------------------- Filters declarations - LoPass 3 Hz
Filter lopass1(3, false);
Filter lopass2(3, false);
Filter lopass3(3, false);
Filter lopass4(3, false);
// ------------------------------------------------- Filters declarations - LoPass 1 Hz for AutoGain
Filter lopass5(2, false);

void loop() 
{  
    sensor.readSensor();

    // -------------------------------------------- Hi Pass - 4 stages
    setFilterInput(sensor.IR);
    hipass1.run();
    hipass2.run();
    hipass3.run();
    hipass4.run();
    // -------------------------------------------- Low Pass - 4 stages
    lopass1.run();
    lopass2.run();
    lopass3.run();  
    lopass4.run();   
    float filtered = getFilterOutput();
    
	// -------------------------------------------- Auto gain
	float v = abs(filtered);
	setFilterInput(v);
	lopass5.run();
	float gain = 24000 / getFilterOutput();
	if (gain > 5000) gain = 5000;
	filtered *= gain;
    
	// -------------------------------------------- Limit amplitude to 16 bits
    filtered += 32768;
    if (filtered > 65535) filtered = 65535;
    if (filtered < 0) filtered = 0;
	
    // -------------------------------------------- Send raw IR and filtered IR to ArduHAL
    Theremino.genericWrite16(0, sensor.IR);
    Theremino.genericWrite16(1, filtered);


    //  TEST, EXAMPLES and DEBUG
    // -------------------------------------------- test data using Theremino-ArduHAL
    //   sensor.IR  = 66666;
    //   sensor.RED = 66667;
    //   sensor.RED = sensor.getPartID();

    // -------------------------------------------- test data using Theremino-Terminal
    //   Serial.print(String(sensor.getPartID()));
    //   Serial.write(13); 
    //   Serial.write(10);
    
    // ----------------------------------
    //  Generic Read and Write examples
    // ----------------------------------   
    //  uint32_t n;
    //  n = Theremino.genericRead8(A2);
    //  Theremino.genericWrite8(A3, n);
    //
    //  n = Theremino.genericRead16(A4);
    //  Theremino.genericWrite16(A5, n);
    //
    //  n = Theremino.genericRead24(A6);
    //  Theremino.genericWrite24(A7, n);
    
    // ----------------------------------------------------------------
    //  Delay example 
    //  a single 10 mS delay decreases drammatically the exchange rate
    // ----------------------------------------------------------------
    // delay(10);
	
	// ----------------------------------------------------------------
    //  Debug the AutoGain sending the gain to the ArduHAL using Pin 9
    // ----------------------------------------------------------------
	// Theremino.genericWrite16(9, gain);

    // ----------------------------------------------------------------
    //  Measure the loop time and send it to the ArduHAL using Pin 10
    // ----------------------------------------------------------------
    // static uint32_t lastUS;
    // uint32_t time = micros();
    // uint32_t elapsedUS = time - lastUS;
    // lastUS = time;
    // Theremino.genericWrite16(10, elapsedUS);

    // ----------------------------------------------------------------
    //  Test times
    // ----------------------------------------------------------------
    // Empty loop = 20 uS
    // 2 x Theremino.genericWrite16 = 4 uS       Tot = 24
    // sensor.readSensor = 940 uS                Tot = 964
    // amplify (1 float multiplication) = 16 uS  Tot = 980
    // limit amplitude (2 IF) = 8 uS             Tot = 988
    // 8 filters (8 x 15 float mult.) = 2000 uS  Tot = 2988
}

